# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright (C) 2012 Thomas Mashos <thomas@mashos.com>
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

from gi.repository import Gio, Gtk, Gdk, Pango, PangoCairo # pylint: disable=E0611
import logging
logger = logging.getLogger('ubuntu_tv_testcard_lib')

import cairo

from . helpers import get_builder, show_uri, get_help_uri
import math

# This class is meant to be subclassed by UbuntuTvTestcardWindow.  It provides
# common functions and some boilerplate.
class Window(Gtk.Window):
    __gtype_name__ = "Window"

    # To construct a new instance of this method, the following notable 
    # methods are called in this order:
    # __new__(cls)
    # __init__(self)
    # finish_initializing(self, builder)
    # __init__(self)
    #
    # For this reason, it's recommended you leave __init__ empty and put
    # your initialization code in finish_initializing
    
    def __new__(cls):
        """Special static method that's automatically called by Python when 
        constructing a new instance of this class.
        
        Returns a fully instantiated BaseUbuntuTvTestcardWindow object.
        """
        builder = get_builder('UbuntuTvTestcardWindow')
        new_object = builder.get_object("ubuntu_tv_testcard_window")
        new_object.finish_initializing(builder)
        return new_object

    def finish_initializing(self, builder):
        """Called while initializing this instance in __new__

        finish_initializing should be called after parsing the UI definition
        and creating a UbuntuTvTestcardWindow object with it in order to finish
        initializing the start of the new UbuntuTvTestcardWindow instance.
        """
        # Get a reference to the builder and set up the signals.
        self.builder = builder
        self.ui = builder.get_ui(self, True)
        self.PreferencesDialog = None # class
        self.preferences_dialog = None # instance
        self.AboutDialog = None # class

        self.settings = Gio.Settings("net.launchpad.ubuntu-tv-testcard")
        self.settings.connect('changed', self.on_preferences_changed)

        # Optional Launchpad integration
        # This shouldn't crash if not found as it is simply used for bug reporting.
        # See https://wiki.ubuntu.com/UbuntuDevelopment/Internationalisation/Coding
        # for more information about Launchpad integration.
        try:
            from gi.repository import LaunchpadIntegration # pylint: disable=E0611
            #LaunchpadIntegration.add_items(self.ui.helpMenu, 1, True, True)
            LaunchpadIntegration.set_sourcepackagename('ubuntu-tv-testcard')
        except ImportError:
            pass

        # Optional application indicator support
        # Run 'quickly add indicator' to get started.
        # More information:
        #  http://owaislone.org/quickly-add-indicator/
        #  https://wiki.ubuntu.com/DesktopExperienceTeam/ApplicationIndicators
        try:
            from ubuntu_tv_testcard import indicator
            # self is passed so methods of this class can be called from indicator.py
            # Comment this next line out to disable appindicator
            self.indicator = indicator.new_application_indicator(self)
        except ImportError:
            pass
        
        self.DrawingArea = self.builder.get_object("DrawingArea")
        self.DrawingArea.connect("draw", self.instructions)
        
        self.connect("key_press_event", self.on_key_press)
        self.screen=Gdk.Screen.get_default()
        
        ## Get info on all monitors
        mon=0
        self.MONITORS=[]
        while mon < self.screen.get_n_monitors():
            x = self.screen.get_monitor_geometry(mon).x
            y = self.screen.get_monitor_geometry(mon).y
            xres = self.screen.get_monitor_geometry(mon).width
            yres = self.screen.get_monitor_geometry(mon).height
            self.MONITORS.append((x,y,xres,yres))
            mon+=1
        
        self.fullscreen()
        self.width = self.screen.get_monitor_geometry(0).width
        self.height = self.screen.get_monitor_geometry(0).height
        self.COLORBG = "#000000"
        self.COLORTEXT = "#FFFFFF"

    def on_mnu_contents_activate(self, widget, data=None):
        show_uri(self, "ghelp:%s" % get_help_uri())

    def on_mnu_about_activate(self, widget, data=None):
        """Display the about box for ubuntu-tv-testcard."""
        if self.AboutDialog is not None:
            about = self.AboutDialog() # pylint: disable=E1102
            response = about.run()
            about.destroy()

    def on_mnu_preferences_activate(self, widget, data=None):
        """Display the preferences window for ubuntu-tv-testcard."""

        """ From the PyGTK Reference manual
           Say for example the preferences dialog is currently open,
           and the user chooses Preferences from the menu a second time;
           use the present() method to move the already-open dialog
           where the user can see it."""
        if self.preferences_dialog is not None:
            logger.debug('show existing preferences_dialog')
            self.preferences_dialog.present()
        elif self.PreferencesDialog is not None:
            logger.debug('create new preferences_dialog')
            self.preferences_dialog = self.PreferencesDialog() # pylint: disable=E1102
            self.preferences_dialog.connect('destroy', self.on_preferences_dialog_destroyed)
            self.preferences_dialog.show()
        # destroy command moved into dialog to allow for a help button

    def on_mnu_close_activate(self, widget, data=None):
        """Signal handler for closing the UbuntuTvTestcardWindow."""
        self.destroy()

    def on_destroy(self, widget, data=None):
        """Called when the UbuntuTvTestcardWindow is closed."""
        # Clean up code for saving application state should be added here.
        Gtk.main_quit()

    def on_preferences_changed(self, settings, key, data=None):
        logger.debug('preference changed: %s = %s' % (key, str(settings.get_value(key))))

    def on_preferences_dialog_destroyed(self, widget, data=None):
        '''only affects gui
        
        logically there is no difference between the user closing,
        minimising or ignoring the preferences dialog'''
        logger.debug('on_preferences_dialog_destroyed')
        # to determine whether to create or present preferences_dialog
        self.preferences_dialog = None

    def on_key_press(self, widget, data=None):
        if not Gdk.keyval_name(data.keyval).isdigit():
            self.erase()
        elif Gdk.keyval_name(data.keyval).isdigit():
            try:
                loc = self.MONITORS[int(Gdk.keyval_name(data.keyval))-1]
                x = loc[0]
                y = loc[1]
                self.width=loc[2]
                self.height=loc[3]
                self.move(x,y)
                getattr(self,self.screenis)(widget)
            except IndexError:
                pass
        if Gdk.keyval_name(data.keyval) == "Escape":
            Gtk.main_quit()
        if not self.screenis == "instructions":
            self.instructions(widget)
        elif Gdk.keyval_name(data.keyval) == "Left":
            self.draw_SMPTE()
        elif Gdk.keyval_name(data.keyval) == "Right":
            self.draw_contrast()
        elif Gdk.keyval_name(data.keyval) == "Up":
            self.draw_aspect_ratio()
        #elif Gdk.keyval_name(data.keyval) == "Down":  ## Disabled until we need a down button
        #    self.test()
        else:
            pass
            
    def erase(self):
        w = self.width
        h = self.height
        cr = self.DrawingArea.get_property('window').cairo_create()
        cr.set_source_rgb(0, 0, 0)
        cr.rectangle(0, 0, w, h)
        cr.fill()

    def instructions(self, widget, data=None):
        self.screenis = "instructions"
        self.erase()
        
        FONT = Pango.FontDescription()
        FONT.set_family("Ubuntu")
		FONT.set_weight(700)
        font_size = 15
        FONT.set_size(font_size * Pango.SCALE)
        
        cr = self.DrawingArea.get_property('window').cairo_create()
        
        r, g, b = self.convert_to_rgb(self.COLORTEXT)
        cr.set_source_rgb(r, g, b)
        
        w = self.width
        h = self.height
        
        layout = PangoCairo.create_layout (cr)
        layout.set_text("Welcome to Ubuntu TV Testcard.\nUse this utility to finetune your TV settings.", -1)
        layout.set_font_description(FONT)

        layout.set_alignment(Pango.Alignment.CENTER)
        PangoCairo.update_layout (cr, layout)
        width, height = layout.get_pixel_size()
        cr.move_to ( (self.width/2)-(width/2), (self.height/2)-(height/2))
        PangoCairo.show_layout (cr, layout)

        arrowsize=25
        cr.set_line_width(9)
        arrowstarty=self.height/2
        
        ## Left arrow
        arrowstartx=50
        
        cr.move_to (arrowstartx,arrowstarty)
        cr.line_to (arrowstartx+arrowsize,arrowstarty-arrowsize)
        cr.line_to (arrowstartx+arrowsize,arrowstarty+arrowsize)
        cr.fill()
        
        layout = PangoCairo.create_layout (cr)
        layout.set_text("Color bars\n(Use to adjust tint)", -1)
        layout.set_font_description( FONT)
        
        layout.set_alignment(Pango.Alignment.CENTER)
        PangoCairo.update_layout (cr, layout)
        width, height = layout.get_pixel_size()
        offset=height/2
        cr.move_to ( arrowstartx+arrowsize+10, arrowstarty-offset)
        PangoCairo.show_layout (cr, layout)
        
        ## Right arrow
        arrowstartx=self.width-50
        
        cr.move_to (arrowstartx,arrowstarty)
        cr.line_to (arrowstartx-arrowsize,arrowstarty-arrowsize)
        cr.line_to (arrowstartx-arrowsize,arrowstarty+arrowsize)
        cr.fill()
        
        layout = PangoCairo.create_layout (cr)
        layout.set_text("Contrast, brightness, and convergence", -1)
        layout.set_font_description(FONT)
        
        layout.set_alignment(Pango.Alignment.CENTER)
        PangoCairo.update_layout (cr, layout)
        width, height = layout.get_pixel_size()
        xoffset=width
        yoffset=height/2
        cr.move_to ( arrowstartx-arrowsize-10-xoffset, arrowstarty-yoffset)
        PangoCairo.show_layout (cr, layout)

        arrowstartx=self.width/2
        ## Top arrow
        arrowstarty=50
        
        cr.move_to (arrowstartx,arrowstarty)
        cr.line_to (arrowstartx-arrowsize,arrowstarty+arrowsize)
        cr.line_to (arrowstartx+arrowsize,arrowstarty+arrowsize)
        cr.fill()
        
        layout = PangoCairo.create_layout (cr)
        layout.set_text("Aspect Ratio, overscan, resolution", -1)
        layout.set_font_description(FONT)
        
        layout.set_alignment(Pango.Alignment.CENTER)
        PangoCairo.update_layout (cr, layout)
        width, height = layout.get_pixel_size()
        offset=width/2
        
        cr.move_to (arrowstartx-offset, arrowstarty+arrowsize+10)
        PangoCairo.show_layout (cr, layout)
        
        ## Bottom arrow
        '''
        arrowstarty=self.height-50
        
        cr.move_to (arrowstartx,arrowstarty)
        cr.line_to (arrowstartx-arrowsize,arrowstarty-arrowsize)
        cr.line_to (arrowstartx+arrowsize,arrowstarty-arrowsize)
        cr.fill()
        
        layout = PangoCairo.create_layout (cr)
        layout.set_text("FILLER", -1)
        layout.set_font_description(FONT)
        
        layout.set_alignment(Pango.Alignment.CENTER)
        PangoCairo.update_layout (cr, layout)
        width, height = layout.get_pixel_size()
        xoffset=width/2
        yoffset=height
        
        cr.move_to (arrowstartx-xoffset, arrowstarty-arrowsize-yoffset-10)
        PangoCairo.show_layout (cr, layout)
        '''

    def draw_aspect_ratio(self, widget=None, data=None):
        self.screenis = "draw_aspect_ratio"
        cr = self.DrawingArea.get_property('window').cairo_create()

        r, g, b = self.convert_to_rgb(self.COLORTEXT)
        cr.set_source_rgb(r, g, b)

        cr.set_line_width(3)

        ## Draw Center Circle
        radius=(self.height/2)/2
        cr.arc(self.width/2, self.height/2, radius, 0, 360*(math.pi/180))
        cr.stroke()

        ## Draw resolution
        FONT = Pango.FontDescription()
        FONT.set_family("Ubuntu")
        font_size = 15
        FONT.set_size(font_size * Pango.SCALE)

        layout = PangoCairo.create_layout (cr)
        layout.set_text("Detected Resolution "+str(self.width)+'x'+str(self.height)+"\n\nThis circle should be round (not oval). If this is not round, your TV is displaying the signal at an incorrect aspect ratio.", -1)
        layout.set_font_description(FONT)
        layout.set_alignment(Pango.Alignment.CENTER)
        layout.set_width(Pango.SCALE * ((radius*2)-50))
        layout.set_wrap(Pango.WrapMode.WORD)
        layout.set_height(Pango.SCALE * radius)
        PangoCairo.update_layout (cr, layout)
        width, height = layout.get_pixel_size()
        xoffset=width/2
        yoffset=height/2

        cr.move_to (self.width/2-xoffset, self.height/2-yoffset)
        PangoCairo.show_layout (cr, layout)

        ## Draw corners
        step=25
        numsteps=5

        i=0
        while i < numsteps:
            cr.rectangle(0, step*i, self.width/2-(step*i), step)
            cr.rectangle(0, step*i, self.width/2-(step*i), step)
            cr.rectangle(step*i, 0, step, self.height/2-(step*i))
            cr.rectangle(self.width-(self.width/2)+step*i, self.height-step*(i+1), self.width/2-(step*i), step)
            cr.rectangle(self.width-step*(i+1), self.height/2+(step*i), step, self.height/2)
            cr.stroke()
            i+=1

        ## Draw info
        layout = PangoCairo.create_layout (cr)
        layout.set_text("Each side of the screen has "+str(numsteps)+" bars that are "+str(step)+" pixels wide. If less than 5 bars are shown, then the edge of the screen is not visable. Check your graphics card and TV settings to disable overscan", -1)
        layout.set_font_description(FONT)
        layout.set_alignment(Pango.Alignment.LEFT)
        layout.set_width(Pango.SCALE * ((self.width/2) - radius - (step*numsteps)))
        layout.set_wrap(Pango.WrapMode.WORD)
        #layout.set_height(Pango.SCALE * radius)
        PangoCairo.update_layout (cr, layout)
        width, height = layout.get_pixel_size()
        xoffset=width/2
        yoffset=height/2

        cr.move_to ((step*numsteps)+25, (step*numsteps)+25)
        PangoCairo.show_layout (cr, layout)

    def draw_contrast(self, widget=None, data=None):
        self.screenis = "draw_contrast"
        
        cr = self.DrawingArea.get_property('window').cairo_create()
        cr.set_line_width(2)
        
        r, g, b = self.convert_to_rgb(self.COLORTEXT)
        cr.set_source_rgb(r, g, b)
        
        #Draw Vertical Lines
        i=20
        while i < self.width:
            cr.move_to(i, 0)
            cr.line_to(i, self.height)
            cr.stroke()
            i+=60
        
        #Draw Horizontal Lines
        i=20
        while i < self.height:
            cr.move_to(0, i)
            cr.line_to(self.width, i)
            cr.stroke()
            i+=60

        # Dray Contrast bars
        colors=['#000000','#0D0D0D','#1A1A1A','#262626','#333333',
                '#404040','#4D4D4D','#595959','#666666','#737373',
                '#7F7F7F','#8A8A8A','#969696','#A3A3A3','#AAAAAA',
                '#B8B8B8','#BFBFBF','#CDCDCD','#D3D3D3','#DEDEDE',
                '#EBEBEB','#F7F7F7','#FFFFFF']
        
        steps=len(colors)
        sectionwidth=int(math.ceil(float(self.width)/float(steps)))

        for colorstring in colors:
            r, g, b = self.convert_to_rgb(colorstring)
            cr.set_source_rgb(r, g, b)
            cr.rectangle(sectionwidth*colors.index(colorstring), (self.height/3)-(sectionwidth*2), sectionwidth, sectionwidth*2)
            cr.fill()

        colors.reverse()

        for colorstring in colors:
            r, g, b = self.convert_to_rgb(colorstring)
            cr.set_source_rgb(r, g, b)
            cr.rectangle(sectionwidth*colors.index(colorstring), (self.height/3)*2, sectionwidth, sectionwidth*2)
            cr.fill()
            
        ## Set Arrow settings
        cr.set_source_rgb(0.75, 0.75, 0.75)
        arrowsize=25

        ## Top left arrow
        startx=sectionwidth
        starty=self.height/3
        cr.move_to (startx,starty)
        cr.line_to (startx-arrowsize,starty+arrowsize)
        cr.line_to (startx+arrowsize,starty+arrowsize)
        cr.fill()
        
        ## Top right arrow
        startx=sectionwidth*(len(colors)-1)
        cr.move_to (startx,starty)
        cr.line_to (startx-arrowsize,starty+arrowsize)
        cr.line_to (startx+arrowsize,starty+arrowsize)
        cr.fill()

        ## Bottom right arrow
        starty=(self.height/3)*2
        cr.move_to (startx,starty)
        cr.line_to (startx-arrowsize,starty-arrowsize)
        cr.line_to (startx+arrowsize,starty-arrowsize)
        cr.fill()
        
        ## Bottom left arrow
        startx=sectionwidth
        cr.move_to (startx,starty)
        cr.line_to (startx-arrowsize,starty-arrowsize)
        cr.line_to (startx+arrowsize,starty-arrowsize)
        cr.fill()

        ## Draw info
        FONT = Pango.FontDescription()
        FONT.set_family("Ubuntu")
        font_size = 15
        FONT.set_size(font_size * Pango.SCALE)
        
        layout = PangoCairo.create_layout (cr)
        layout.set_text("Ensure you can see the difference between the two whitest and blackest bars (boundary at the arrows). Adjust the TV brightness and Contrast settings accordingly", -1)
        layout.set_font_description(FONT)
        layout.set_alignment(Pango.Alignment.CENTER)
        frame=20
        layout.set_width(Pango.SCALE * (self.width/2))
        
        layout.set_wrap(Pango.WrapMode.WORD)
        #layout.set_height(Pango.SCALE * radius)
        PangoCairo.update_layout (cr, layout)
        width, height = layout.get_pixel_size()
        xoffset=width/2
        yoffset=height/2

        cr.set_source_rgb(0.75, 0.75, 0.75)
        cr.rectangle(self.width/2-xoffset-frame, self.height/2-yoffset-frame, (xoffset*2)+(frame*2), (yoffset*2)+(frame*2))
        cr.fill()
        
        r, g, b = self.convert_to_rgb(self.COLORTEXT)
        cr.set_source_rgb(r, g, b)
        cr.move_to (self.width/2-xoffset-frame, self.height/2-yoffset)
        PangoCairo.show_layout (cr, layout)

    def draw_SMPTE(self, widget=None, data=None):
        self.screenis = "draw_SMPTE"

        cr = self.DrawingArea.get_property('window').cairo_create()

        colors=["#BFBFBF","#bfbf00","#00bfbf","#00bf00","#bf00bf","#bf0000","#0000c1"]

        centersection=int((float(self.width)/float(4))*3)
        centersectionsmall=int(math.ceil(float(centersection)/float(7)))
        outsidesection=(self.width-centersection)/2
        sectionheight=int(math.ceil(float(self.height)/float(12)))

        ## Pattern 1
        r, g, b = self.convert_to_rgb("#666666")
        cr.set_source_rgb(r, g, b)
        cr.rectangle(0, 0, outsidesection, sectionheight*7)
        cr.rectangle(self.width-outsidesection, 0, outsidesection, sectionheight*7)
        cr.fill()
        i=0        
        for c in colors:
            r, g, b = self.convert_to_rgb(c)
            cr.set_source_rgb(r, g, b)
            cr.rectangle(outsidesection+(centersectionsmall*i), 0, centersectionsmall, sectionheight*7)
            cr.fill()
            i+=1

        ## Pattern 2
        r, g, b = self.convert_to_rgb("#00FFFF")
        cr.set_source_rgb(r, g, b)
        cr.rectangle(0, sectionheight*7, outsidesection, sectionheight*1)
        cr.fill()
        
        r, g, b = self.convert_to_rgb("#0000c1")
        cr.set_source_rgb(r, g, b)
        cr.rectangle(self.width-outsidesection, sectionheight*7, outsidesection, sectionheight*1)
        cr.fill()
        
        r, g, b = self.convert_to_rgb("#FFFFFF")
        cr.set_source_rgb(r, g, b)
        cr.rectangle(outsidesection, sectionheight*7, centersectionsmall, sectionheight)
        cr.fill()
        
        r, g, b = self.convert_to_rgb("#BFBFBF")
        cr.set_source_rgb(r, g, b)
        cr.rectangle(outsidesection+centersectionsmall, sectionheight*7, centersectionsmall*6, sectionheight)
        cr.fill()

        ## Pattern 3
        r, g, b = self.convert_to_rgb("#ffff00")
        cr.set_source_rgb(r, g, b)
        cr.rectangle(0, sectionheight*8, outsidesection, sectionheight*1)
        cr.fill()

        r, g, b = self.convert_to_rgb("#ff0000")
        cr.set_source_rgb(r, g, b)
        cr.rectangle(self.width-outsidesection, sectionheight*8, outsidesection, sectionheight*1)
        cr.fill()

        lg = cairo.LinearGradient(outsidesection, 0.0, centersectionsmall*7, 0.0)
        lg.add_color_stop_rgb(0, 0, 0, 0) 
        lg.add_color_stop_rgb(1, 1, 1, 1) 
        cr.rectangle(outsidesection, (sectionheight*8), centersectionsmall*7, sectionheight)
        cr.set_source(lg)
        cr.fill()

        ## Pattern 4
        r, g, b = self.convert_to_rgb("#262626")
        cr.set_source_rgb(r, g, b)
        cr.rectangle(0, sectionheight*9, outsidesection, sectionheight*3)
        cr.rectangle(self.width-outsidesection, sectionheight*9, outsidesection, sectionheight*3)
        cr.fill()
        
        startloc=outsidesection
        r, g, b = self.convert_to_rgb("#050505")
        cr.set_source_rgb(r, g, b)
        cr.rectangle(outsidesection, sectionheight*9, int(float(centersectionsmall)*1.5), sectionheight*3)
        cr.fill()

        startloc+=int(float(centersectionsmall)*1.5)
        r, g, b = self.convert_to_rgb("#FFFFFF")
        cr.set_source_rgb(r, g, b)
        cr.rectangle(startloc, sectionheight*9, int(float(centersectionsmall)*2), sectionheight*3)
        cr.fill()
        
        startloc+=int(float(centersectionsmall)*2)
        r, g, b = self.convert_to_rgb("#050505")
        cr.set_source_rgb(r, g, b)
        cr.rectangle(startloc, sectionheight*9, int((float(centersectionsmall)/6)*5), sectionheight*3)
        cr.fill()
        
        startloc+=int((float(centersectionsmall)/6)*5)
        r, g, b = self.convert_to_rgb("#000000") ## should be -2% of black (super black)
        cr.set_source_rgb(r, g, b)
        cr.rectangle(startloc, sectionheight*9, int(float(centersectionsmall)/3), sectionheight*3)
        cr.fill()
        
        startloc+=int(float(centersectionsmall)/3)
        r, g, b = self.convert_to_rgb("#050505")
        cr.set_source_rgb(r, g, b)
        cr.rectangle(startloc, sectionheight*9, int(float(centersectionsmall)/3), sectionheight*3)
        cr.fill()
        
        startloc+=int(float(centersectionsmall)/3)
        r, g, b = self.convert_to_rgb("#0A0A0A")
        cr.set_source_rgb(r, g, b)
        cr.rectangle(startloc, sectionheight*9, int(float(centersectionsmall)/3), sectionheight*3)
        cr.fill()
        
        startloc+=int(float(centersectionsmall)/3)
        r, g, b = self.convert_to_rgb("#050505")
        cr.set_source_rgb(r, g, b)
        cr.rectangle(startloc, sectionheight*9, int(float(centersectionsmall)/3), sectionheight*3)
        cr.fill()
        
        startloc+=int(float(centersectionsmall)/3)
        r, g, b = self.convert_to_rgb("#0F0F0F")
        cr.set_source_rgb(r, g, b)
        cr.rectangle(startloc, sectionheight*9, int(float(centersectionsmall)/3), sectionheight*3)
        cr.fill()

        startloc+=int(float(centersectionsmall)/3)
        r, g, b = self.convert_to_rgb("#050505")
        cr.set_source_rgb(r, g, b)
        cr.rectangle(startloc, sectionheight*9, (outsidesection+(centersectionsmall*7))-startloc, sectionheight*3)
        cr.fill()

    def convert_to_rgb(self, HTML):
        if HTML[0] == '#':
            cstring = HTML[1:]
        else:
            cstring = HTML
        r, g, b = cstring[:2], cstring[2:4], cstring[4:]
        r, g, b = [int(n, 16) for n in (r, g, b)]
        red = float(r)/float(255)
        green = float(g)/float(255)
        blue = float(b)/float(255)
        return red, green, blue
        
